function [ corrected ] = correctdistortion( measurements, locationcoeffs, anglecoeffs, deg)
%CORRECTDISTORTION performs distortion compensation on magnetic tracker
%readings.
%
%  CORRECTED = CORRECTDISTORTION(MEAS,LOCATIONCOEFFS,ANGLECOEFFS,DEG)
%  Where MEAS is a set of n magnetic tracker readings to be corrected
%  (represented as an nx6 matrix with columns for x, y and z location
%  of the magnetic tracker as well as azimuth, elevation and roll
%  Euler angles).
%
%  LOCATIONCOEFFS and ANGLECOEFFS are polynomial coefficients modeling the
%  shape of the distortion. These coefficients can be generated with the
%  function FINDCORRECTIONCOEFFS.
%
%  DEG is the degree of the polynomial to use for the correction procedure.
%
%  Returns CORRECTED, a matrix of the same size as MEAS containing
%  distortion-compensated coordinates of corresponding entries in MEAS.

%Copyright 2004 James Barabas.
%jdb30@cornell.edu

if (~isempty(locationcoeffs))
    locations = correctlocations(locationcoeffs,measurements(:,1:3),deg);
else 
    locations = measurements(:,1:3);
end

if (~isempty(anglecoeffs))
    angles = correctangles(anglecoeffs, measurements(:,1:3),measurements(:,4:6),deg);
else
    angles = measurements(:,4:6);
end

corrected = [locations,angles,measurements(:,7:end)];





%_____________________________________________________________________
function ret = correctlocations(P, dataset,degree)
%correctlocations(P, dataset,degree)
%takes a set of coeffs and a tracker dataset (3 positions & 3 orientations each reading row) 
%and corrects the dataset using the coeffs in the quadratic below.
%P is a matrix of coeffs for the rescaling equation. For the 4th degree case, this requires 35*3=105 coefs.
%
%P is an nx3 matrix, where first column is data for polynomial x axis error, 2nd is y, 3rd is z. 

%Solve for solution to Ax=b for coeff vector x.
nummeas = size(dataset,1);
coefs = numcoeffs(degree);


x = zeros (coefs);

A = zeros (nummeas,coefs);

b = zeros (nummeas,1);

%r = zeros (nummeas,3);

x = dataset(:,1);
y = dataset(:,2);
z = dataset(:,3);

index = 1;

%all terms where sum of exponents of x,y,z is at most degree.
for i=0:degree
    for j = 0:i
        for k = 0:(i-j)
            A(:,index) = x.^j .* y.^k .* z.^((i-j)-k);
            index = index+1;
        end
    end
end

ret = [A*P(:,1) + x, A*P(:,2) + y, A*P(:,3) + z, dataset(:,4:size(dataset,2))];

%_____________________________________________________________________
function ret = correctangles(P,positions,angles,degree)
%correctangles(P,positions,angles,degree)

%Solve for solution to Ax=b for coeff vector x.
nummeas = size(angles,1);
coefs = numcoeffs(degree);


x = zeros (coefs);

A = zeros (nummeas,coefs);

b = zeros (nummeas,1);

%r = zeros (nummeas,3);

x = positions(:,1);
y = positions(:,2);
z = positions(:,3);

az = angles(:,1);
el = angles(:,2);
ro = angles(:,3);

index = 1;

%all terms where sum of exponents of x,y,z is at most degree.
for i=0:degree
    for j = 0:i
        for k = 0:(i-j)
            A(:,index) = x.^j .* y.^k .* z.^((i-j)-k);
            index = index+1;
        end
    end
end

ret = [A*P(:,1) + az, A*P(:,2) + el, A*P(:,3) + ro, angles(:,4:size(angles,2))];

%__________________________________________________________________________
function p = numcoeffs(degree)
%PARAMS = NUMPARAMS(DEGREE)
% find number of correction parameters for a distortion correction
% polynomial of degree specified.

p = (degree+1)*(degree+2)*(degree+3)/6;
