%FINDCORRECTIONCOEFFS computes coefficients of a set of polynomial
%equations for correcting for magnetic distortion in magnetic tracker measurements. 
%
%  [LOCATONCOEFFS, ANGLECOEFFS] = FINDCORRECTIONCOEFFS(MEASURED, ACTUAL, DEGREE)
%  MEASURED is an n x 6 array containing readings from a magnetic tracker.
%  ACTUAL is an n x 6 array containing ground-truth locations
%  of the tracker sensor corresponding to the readings in MEASURED.
% 
%  The 6 columns of MEASURED and ACTUAL are x, y and z coordinates of location of 
%  the magnetic tracker as well as azimuth, elevation and roll Euler angles.
% 
%  The parameter DEGREE specifies the degree of the polynomial equation to
%  be used for distortion compensation. 
%
%  FINDCORRECTIONCOEFFS returns two arrays of coefficients: LOCATIONCOEFFS
%  and ANGLECOEFFS. Each of these arrays is of size m x 3, where
%  m = (DEGREE+1)*(DEGREE+2)*(DEGREE+3)/6.
%  Each column of LOCATIONCOEFFS contains the set of coefficients
%  for one of the three location coordinates (x, y ,z). Likewise, the columns of
%  ANGLECOEFFS contain coefficients for correcting each of the three Euler
%  angles (azimuth, elevation, roll) in a tracker reading.
%  Rows of these matrices contain coefficients of nondecreasing degree. (The
%  first row contains bias terms, followed by linear terms in the second
%  row.)
%
%  Use coefficients returned by FINDCORRECTIONCOEFFS with the 
%  function CORRECTDISTORTION to perform distortion compensation.

%Copyright 2004 James Barabas.
%jdb30@cornell.edu

function [lc,ac] = findcorrectioncoeffs(measured, actual, degree)
    
lc = findlocationcorrection(measured, actual, degree);
ac = findanglecorrection(measured, actual, degree);


%_____________________________________________________________________
function R = findlocationcorrection(measured, actual, degree)
%COEFFS = FINDLOCATIONCORRECTION(MEASURED, ACTUAL, DEGREE)
%given measured and actual position data, returns coeffs of the polynomial for computing the displacement
%due to magnetic distortion at a point in space.
%
%computes polynomial representing error. Add to measured data for "correct"
%data.

%Solve for solution to Ax=b for coeff vector x.
measuredlocations = measured(:,1:3);
actuallocations = actual(:,1:3);
nummeas = size(measuredlocations,1);
A = zeros(nummeas,numcoeffs(degree));

x = measuredlocations(:,1);
y = measuredlocations(:,2);
z = measuredlocations(:,3);

index = 1;

%all terms where sum of exponents of x,y,z is at most degree.
for i=0:degree
    for j = 0:i
        for k = 0:(i-j)
            A(:,index) = x.^j .* y.^k .* z.^((i-j)-k);
            index = index+1;
        end
    end
end

b = actuallocations - measuredlocations;

R(:,1) = A\b(:,1);
R(:,2) = A\b(:,2);
R(:,3) = A\b(:,3);


%display residuals (if dumps big numbers, we have a bad fit)
%resid = (A*R(:,1)-b)
%resid = (A*R(:,2)-b)
%resid = (A*R(:,3)-b)
%_____________________________________________________________________

function R = findanglecorrection(measured, actual, degree)
%COEFFS = FINDANGLECORRECTION(MEASURED, ACTUAL, DEGREE)
%given measured and actual position & orientation data, returns coeffs 
%of the polynomial for computing the angular displacement
%due to magnetic distortion at a point in space.
%
%computes polynomial representing error. Add to measured data for "correct" data.


measuredangles = measured(:,4:6);
measuredlocations = measured(:,1:3);
actualangles = actual(:,4:6);

%Solve for solution to Ax=b for coeff vector x.
nummeas = size(measuredangles,1);
A = zeros (nummeas,numcoeffs(degree));

x = measuredlocations(:,1);
y = measuredlocations(:,2);
z = measuredlocations(:,3);

index = 1;

%all terms where sum of exponents of x,y,z is at most degree.
for i=0:degree
    for j = 0:i
        for k = 0:(i-j)
            A(:,index) = x.^j .* y.^k .* z.^((i-j)-k);
            index = index+1;
        end
    end
end

b = actualangles(:,1:3) - measuredangles(:,1:3);

R(:,1) = A\b(:,1);
R(:,2) = A\b(:,2);
R(:,3) = A\b(:,3);


%display residuals (if dumps big numbers, we have a bad fit)
%resid = (A*R(:,1)-b)
%resid = (A*R(:,2)-b)
%resid = (A*R(:,3)-b)

%__________________________________________________________________________
function p = numcoeffs(degree)
%PARAMS = NUMPARAMS(DEGREE)
% find number of correction parameters for a distortion correction
% polynomial of degree specified.

p = (degree+1)*(degree+2)*(degree+3)/6;
