function [ params] = findmodelparams(guess, meas, targets)
%FINDMODELPARAMS Perform calibration for tracking the Line of Primary Gaze. 
%Find a set of model parameters that minimize error
%between model-predicted screen intersection and actual screen location
%of calibration targets
%
% PARAMS = FINDMODELPARAMS(GUESS, MEAS, TARGETS) returns the 12 parameters
% computed from the set of tracker measurements MEAS, the set of screen
% target locations TARGETS and an initial guess of the parameters GUESS.
%
% The parameter MEAS, the set of head-tracker readings, is an N by 6 array
% where n is the number of readings. The columns of N are the x, y and z
% coordinates followed by the azimuth, elevation and roll angles of the
% tracker readings.
%
% The parameter TARGETS, the set of screen-target locations in the screen
% coordinate frame. This parameter is an N by 2 array, where N is the
% number of rows in MEAS. The columns of TARGETS are the x and y
% coordinates of the screen intersections.
%
% The parameter GUESS, the initial guess of the model parameters is a 1 by
% 12 array. The parameters in this array represent the x, y and z, azimuth,
% elevation and roll components of the Tob and Tse coordinate
% transformations. See SCREENINTERSECTIONS for more details.

%Copyright 2004 James Barabas.
%jdb30@cornell.edu

%get number of tracker measurements
n = size(meas,1);

NumParams = 12;

%check parameters
if (size(meas,2) < 6)
    error('Parameter MEAS must contain 6 columns');
end

if (size(targets,2) ~= 2)
    error('Parameter TARGETS must contain 2 columns');
end

if (size(targets,1) ~= n)
    error('Parameters MEAS and TARGETS must contain the same number of rows');
end

%tolerate transposed guess parameter
if( all(size(guess) == [1,NumParams]))
    guess = guess';
else
    if( ~all(size(guess) == [NumParams,1]))
        error(sprintf('Parameter GUESS should be a %d by 1 array',NumParams));
    end
end

%precompute matrix form of tracker measurements
for(k=1:n)
    measmtx(:,:,k) = trackereulerstomtx4(meas(k,:));
end

%collapse measurements into internally-used format as nlinfit takes
%data only a single 2 dimensional matrix data argument. 
measmtxflat = reshape(measmtx,n,16);

%Put the screen target locations in a global variable.
%This is done to work around nlinfit which only supports functions
%with a single 2-D argument.
global GLOBALLOPGTARGETS;
GLOBALLOPGTARGETS = targets;


%perform fitting: find parameters that give screen-intersection errors 
%that across all measurements are closest to zero, given initial guess 
[params,resid,J] = nlinfit(measmtxflat,zeros(n,1),@intersecterror, guess);



