function errors = intersecterror(P,meas,targets)
%Find the on-screen error distances between targets and screen intersections
%computed from magnetic tracker readings and LoPG-model parameters.
%
%  ERRORS = INTERSECTERROR(PARAMS, MEASUREMENTS, TARGETS)
%  Finds the scalar error distance between predicted and
%  actual screen intersection of the line of primary gaze.
%
%  PARAMS is vector of 12 model parameters (see SCREENINTERSECTIONS).
%
%  The parameter MEAS, the set of head-tracker readings, is an N by 6 array
%  where n is the number of readings. The columns of N are the x, y and z
%  coordinates followed by the azimuth, elevation and roll angles of the
%  tracker readings.
%
%  The parameter TARGETS, the set of screen-target locations in the screen
%  coordinate frame. This parameter is an N by 2 array, where N is the
%  number of rows in MEAS. The columns of TARGETS are the x and y
%  coordinates of the screen intersections.

%Copyright 2004 James Barabas.
%jdb30@cornell.edu

%extract targets from global variable
global GLOBALLOPGTARGETS;

if(exist('targets'))
    GLOBALLOPGTARGETS=targets;
end


%Get x-y screen intersection from data and model parameters
predictedintersects = screenintersections(P,meas);



%find scalar distance between computed and actual intersection
sqxandy = (GLOBALLOPGTARGETS - predictedintersects).^2;

%for nlinfit
errors = sqrt(sqxandy(:,1) + sqxandy(:,2)); 
%disp(mean(errors));

%for fminsearch
%errors = mean(sqrt(sqxandy(:,1) + sqxandy(:,2)));
%disp(errors);


function I = screenintersections(params, meas)
%Use model of Line of Primary Gaze to find point of gaze on screen.
%
%  I = SCREENINTERSECTIONS(PARAMS, MEAS) returns I, an array of x-y points of intersection 
%  of the Line of Primary Gaze with the screen. Intersections are computed
%  for  MEAS, a set of magnetic tracker readings and PARAMS, a set of
%  model parameters.
%
%  MEAS is a set of n measurements of relationships between
%  Broadcaster to Sensor frames taken from magnetic tracker (Msb).
%  This variable may be provided in one of three formats:
%
%  As an Nx6 array where each row contains the x, y, z, azimuth,
%     elevation and roll variables of the tracker measurement.
%
%  As a 4x4xN array where each plane in the third dimension is a
%     4x4 transformation matrix representation of Msb as 
%     computed by TRACKEREULERSTOMTX4.
%
%  As an array containing reshaped transformation matrix data. This
%     format is used with the nlinfit function which only 
%     supports 2-dimensonal arrays. To use this format, reshape 
%     a 4x4xN array of transfromation matrices to Nx16.
%
%  PARAMS is a column vector containing 12 model parameters:
%  PARAMS(1:6) is relationship between Sensor & Laser frames (Mse)
%  PARAMS(7:12) is relationship between Origin to Broadcaster (Mob)

%Copyright 2004 James Barabas.
%jdb30@cornell.edu

params = params(:); %make params into a column vector

%See if the MEAS param was a set of 6-vectors or 4x4 matrices
if (ndims(meas) == 3) %MEAS is a set of matrices
    n = size(meas,3); %number of intersections to find
    meas3d = meas;
    fast = 1;
elseif (size(meas,2) == 16) %MEAS is a set of 4-matrices collapsed for nlinfit
    n = size(meas,1);
    meas3d = reshape(meas,4,4,n);
    fast = 1;
else     %MEAS is a set of vectors
    n = size(meas,1); %number of intersections to find
    fast = 0;
end

%use displacements and angles to make a 4x4 matrix representing the transform 
%from Laser to Sensor systems.

%six-parameter description of sensor-eye transform
Mse = params(1:6);

%six-parameter description of broadcaster-origin transform
Mbo = params(7:12);

%eye-sensor transform (invariant of tracker measurement)
Tse = trackereulerstomtx4(Mse);

%broadcaster-origin transform (invariant of tracker measurement)
Tob = trackereulerstomtx4(Mbo);   

%for each tracker measurement
for c = 1:n 
    if (fast)
        %retrieve precomputed broadcaster-sensor transform
        Tsb = meas3d(:,:,c);
    else
        %assemble broadcaster-sensor transform from euler angles
        Tsb = trackereulerstomtx4(meas(c,:));    
    end
    
    %combine all transformations in order
    Toe = Tob*Tsb*Tse;
    
    %compute screen intersection by solving for point on x axis of
    %frame e that is in the x-y plane of frame o.
    I(c,1) = Toe(1,4) - Toe(1,1)*Toe(3,4)/Toe(3,1);
    I(c,2) = Toe(2,4) - Toe(2,1)*Toe(3,4)/Toe(3,1);
    
    
end
