function [ M ] = trackereulerstomtx4(meas)
%Finds a 4x4 homogeneous transformation matrix corresponding 
%to a given displacement, Azumuth, Elevation & Roll.
%
%M = TRACKEREULERSTOMTX4(MEAS)
%
%Parameter MEAS is a 6-vector comprising one Euler-convention reading from
%the magnetic tracker. The six values in this vector are x, y and z displacements
%followed by azimuth, elevation and roll Euler orientation angles.
%All angles are in degrees.

%Copyright 2004 James Barabas.
%jdb30@cornell.edu


%Coordinate system used is right handed.

%Euler angles are applied as follows:
%first, roll is applied about the x axis
%then elevation about the y axis
%finally, azimuth is rotation about the z axis

%This function creates the transformation matrix by composing three
%rotation matrices -- one for each of Roll, Elevation, and Azimuth --
%and one displacement matrix.

meas = meas(:);

%extract displacements
Xo = meas(1);
Yo = meas(2);
Zo = meas(3);

%Convert Euler angles to radians
eulers = meas(4:6).*pi./180;

%precompute sines and cosines of euler angles
s = sin(eulers);    c = cos(eulers);

sa = s(1);          ca = c(1);
se = s(2);          ce = c(2);
sr = s(3);          cr = c(3);

%compose the three rotations and add the displacement

%LONG FORM:
%Rotation about x axis
%Q = [ 1,   0,  0,  0;    0, cr, -sr,  0;     0, sr, cr,  0;   0, 0, 0, 1];
%Rotation about y axis
%R = [ce,   0, se,  0;    0,  1,   0,  0;   -se,  0, ce,  0;   0, 0, 0, 1];
%Rotation about z axis
%S = [ca, -sa,  0,  0;   sa, ca,   0,  0;     0,  0,  1,  0;   0, 0, 0, 1];
%Displacement
%T = [ 1,   0,  0, Xo;    0,  1,   0, Yo;     0,  0,  1, Zo;   0, 0, 0, 1];
%compose the three rotation matrices  
%M = T*S*R*Q;  
%END LONG FORM


%CONDENSED FORM:
%The above four matrices Q, R, S and T can be condensed 
%   into the following single matrix
M = [       ca*ce, -sa*cr+ca*se*sr,  sa*sr+ca*se*cr,               Xo;
            sa*ce,  ca*cr+sa*se*sr, -ca*sr+sa*se*cr,               Yo;
              -se,           ce*sr,           ce*cr,               Zo;
                0,               0,               0,               1];




